@@ -12,6 +12,7 @@
 /*                                                                       */
 /*************************************************************************/
 
+#include <limits.h>
 #include <math.h>
 #include <string.h>
 #include <link-grammar/api.h>
@@ -859,18 +860,13 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 	overflowed = build_parse_set(sent, sent->null_count, opts);
 	print_time(opts, "Built parse set");
 
-	if (overflowed)
+	if (overflowed && (1 < opts->verbosity))
 	{
-		/* We know that sent->num_linkages_found is bogus, possibly negative */
-		sent->num_linkages_found = opts->linkage_limit;
-		if (1 < opts->verbosity)
-		{
-			err_ctxt ec;
-			ec.sent = sent;
-			err_msg(&ec, Warn, "Warning: Count overflow.\n"
-			  "Considering a random subset of %d of an unknown and large number of linkages\n",
-				opts->linkage_limit);
-		}
+		err_ctxt ec;
+		ec.sent = sent;
+		err_msg(&ec, Warn, "Warning: Count overflow.\n"
+		  "Considering a random subset of %d of an unknown and large number of linkages\n",
+			opts->linkage_limit);
 	}
 	N_linkages_found = sent->num_linkages_found;
 
@@ -1232,11 +1228,12 @@ static void chart_parse(Sentence sent, Parse_Options opts)
 {
 	int nl;
 
+	init_count(sent);
+
 	/* Build lists of disjuncts */
 	prepare_to_parse(sent, opts);
 
 	init_fast_matcher(sent);
-	init_table(sent);
 
 	/* A parse set may have been already been built for this sentence,
 	 * if it was previously parsed.  If so we free it up before
@@ -1251,16 +1248,31 @@ static void chart_parse(Sentence sent, Parse_Options opts)
 		sent->null_count = nl;
 		total = do_parse(sent, sent->null_count, opts);
 
+		if (verbosity > 1)
+		{
+			prt_error("Info: Total count with %d null links:   %lld\n",
+				sent->null_count, total);
+		}
+
 		/* Give up if the parse count is overflowing */
-		if (PARSE_NUM_OVERFLOW < total) break;
+		if (PARSE_NUM_OVERFLOW < total)
+		{
+			if (verbosity > 0)
+			{
+				prt_error("WARNING: Combinatorial explosion! nulls=%d cnt=%lld\n"
+					"Consider retrying the parse with the max allowed disjunct cost set lower.\n",
+					sent->null_count, total);
+			}
+			total = (total>INT_MAX) ? INT_MAX : total;
+		}
 
 		sent->num_linkages_found = (int) total;
 		print_time(opts, "Counted parses");
+
 		post_process_linkages(sent, opts);
 		if (sent->num_valid_linkages > 0) break;
 	}
 
-	free_table(sent);
 	free_fast_matcher(sent);
 }
 
@@ -1298,7 +1310,6 @@ int sentence_parse(Sentence sent, Parse_Options opts)
 	}
 
 	init_analyze(sent);
-	init_count(sent);
 
 	/* Expressions were previously set up during the tokenize stage. */
 	expression_prune(sent);
